function [RMSEC_CV,RecSS,PreSS,testTemps]=CPDScv(xSpectra,yCal,Temps,TNOcv,Tcal,stds,model,PolOrder,WinSize,PDSlvs)
%global spektranew,  spektranew = [ ];


%   [RMSEC_CV,RecSS,PreSS,testTemps]=CPDScv(xSpectra,yCal,Temps,TNOcv,Tcal,stds,model,PolOrder,WinSize,PDSlvs);
%
%   Purpose:    Optimisation of meta-parameters within CPDS transfer by crossvalidation.
%               The meta-parameters are:    a) PDS window size                  (omega)
%                                           b) PDS internal latent variables    (nu)
%                                           c) polynomial order                 (eta)
%               The routine NEEDS the stdgen routine from the PLS version 2.0 toolbox.
%               Note that this specific routine is protected by copyright:
%                   Copyright Eigenvector Research, Inc. 1994-98
%
%   Parameters:
%   xSpectra      - (3D tensor) with spectra of calibration samples. The standardisation samples
%                   are a subset of the calibration samples.
%                   first dimension:    samples
%                   second dimension:   channels (wavelengths)
%                   third dimension:    temperatures
%                   Example:    xSpectra(:,:,1) resp. xSpectra(:,:,2) gives spectra at 30oC resp. 40oC.  
%   yCal        -   (Vector) containing the reference values of the calibration samples. 
%   Temps       -   (vector) with temperatures at which calibration samples are measured.
%                   Example:    Temps = [30 40 50 60 70] means that the spectra xSpectra(:,:,1)
%                               are measured at 30oC and xSpectra(:,:,2) at 40oC, etc. 
%   TNOcv       -   (Vector) containing the temperatures in 'Temps' that should NOT be
%                   used in the cross-validation. An empty vector thus indicates that
%                   all temperatures should be used.
%                   Example:    TNOcv = [30 70] will NOT crossvalidate over the
%                               temperatures: 30 and 70 degrees Celsius. This can
%                               e.g. be done to avoid polynomial extrapolation. 
%   Tcal        -   (scalar) the calibration temperature.
%                   Example:    Tcal = 50.
%   stds        -   (vector) Denoting the indexes of the standardisation samples (first dimension
%                   of the matrix 'xSpectra').
%                   Example:    stds = [1,2] means that the matrix xSpectra(1,:,:) and matrix
%                               xSpectra(2,:,:) are the spectra for each temperature (contained in
%                               'Temps')of standardisation samples 1 and 2 respectively.
%   model       -   (struct) with info anbout multivariate model at the calibration temperature.
%                   .bvec       Regression vector
%                   .mncn       1 = mean centering was used. 0 = no mean centering used.
%                   .auto       1 = auto scaling was used.   0 = no auto scaling used.       
%
%   The range of the metaparameters over which to cross-validate are defined by:
%   PolOrder    -   (vector) The polynomial orders to use 
%                   Example:    PolOrder = [1,2,3] 
%   WinSize     -   (vector) The PDS window size
%                   Example:    WinSize  = [3,5,7]; 
%   PDSlvs      -   (vector) The numbers of PDS internal latent variables to use.
%                   Example:    PDSlvs  = [1,2,3];
%
%
%   Returned Parameters:
%   RMSEC_CV    -   3-D tensor containing the root mean square errors of cross-validation.
%                   over all left out temperatures and standardization samples: 
%                   First dimension     :   RMSEC_CV for each internal number of PDS latent
%                                           variables.
%                   Second dimension    :   RMSEC_CV for each PDS window size. 
%                   Third dimension     :   RMSEC_CV for each polynomial order.
%   RecSS       -   5-D tensor containing the spectral reconstruction errors.
%                   The reconstruction errors as single values, the meaning of the dimensions are:
%                   First dimension     :   Reconstruction error for each internal number of PDS
%                                           latent variables tried.
%                   Second dimension    :   Reconstruction error for each PDS window size tried.
%                   Third dimension     :   Reconstruction error for each polynomial order used.
%                   Fourth dimension    :   Reconstruction error for each left out temperature in
%                                           crossvalidation.
%                   Fifth dimension     :   Reconstruction error for each left out standardization
%                                           sample.
%   PreSS       -   5-D tensor containing the squared difference between predicted reference values
%                   and the reference values supplied in 'yCal'. Dimensions have the same meaning as
%                   for the 'RecSS' tensor.
%   testTemps   -   Array with the test temperatures used.
%                   
%
%   Description:
%   ------------
%   This routine performs the crossvalidation procedures as described in detail in the
%   reference below. Normally the calibration temperature whould be the temperature
%   in approx. the center of the temperature range.
%
%   Because no transfer is needed at the calibration temperature. This temperature is
%   not used for crossvalidation.
%
%   A remark concerning the range in which a meta parameter is varied:
%       If an initial estimation of the peak shift in the spectrum caused by temperature
%       change is n channels, then the window size should be set to 9 (2*n+1). This is
%       due to symmetric window that is used here.
%       The WinSize vector should then contain window sizes arround 9, 
%       viz. WinSize=[5 7 9 11 13].
%
%   Example of use:
%   ---------------
%   See the script: CPDSexample.m
%
%   Authors:
%   --------
%   Hans Barring & Hans Boelens
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
%
%   Reference:
%   ----------
%   Optimizing Meta-parameters in Continuous Piecewise Direct Standardization
%   Hans K. Barring, Hans F.M. Boelens, Onno E. de Noord, Age K. Smilde
%   Applied Spectroscopy, accepted december 2000.
%

%   ==================================================
%   =========   Argument Checking ====================
%   ==================================================
[mi,mj,mk]=size(xSpectra);
if ( mk < 3 )
   error('xSpectra is a 3-D tensor, (samples x channels x temperatures) ; At least 3 temperatures are needed')
end

% Make yCal a column vector.
if ( min(size(yCal)) > 1 )
    error('yCal should be a vector')
end
yCal = yCal(:);     

if ( length(yCal) ~= mi )
    error('first dimension of xSpectra and yCal must be identical (= # samples)')     
end

% Make Temps a row vector.
if ( min(size(Temps)) > 1 )
    error('Temps should be a vector')
end
Temps = Temps(:)';  
if ( length(Temps) ~=  mk ) 
    error('# of temperature in ''Temps'' should match third dimension of ''xSpectra''')
end

% Make TNOcv a row vector.
if ( min(size(TNOcv)) > 1 )
    error('''TNOcv'' should be a vector')
end
TNOcv = TNOcv(:)';      
if ( ~isempty( setdiff(TNOcv,Temps )) )
    error('''TNOcv'' should only contain temperatures that are in ''Temps''')
end

% Make Tcal should be a scalar.
if ( max(size(Tcal)) > 1 )
    error('''Tcal'' should be a scalar')
end
if ( isempty(find( Temps == Tcal ) )  )
    error('''Tcal'' should by in ''Temps''')
end

% Make stds a row vector.
if ( min(size(stds)) > 1 )
    error('''stds'' should be a vector')
end
stds = stds(:)';
if ( any(~ismember(stds,1:mi)) )
    error(sprintf('''stds'' should be in range of calibration samples 1..%d',mi))
end

% Make model a column vector.
if ( ~isstruct(model) )
    if ( max(size(model)) > 1 )
        % If a vector is passed, assume that only regression vector is passed and
        % set mean centrering autoscaling to default values. This is: assume mean centering.
        model.bvec = model(:);
        model.mncn = 1;
        model.auto = 0;
    else
        error('''model'' should be a struct or a vector')
    end
else
    % Else make the regression vector a column vector and check length
    model.bvec = model.bvec(:);
end    

if ( length(model.bvec) ~= mj )
    error('length of model regression vector should equal length second dimension of xSpectra') 
end

% Make PolOrder a row vector.
if ( min(size(PolOrder)) > 1 )
    error('''PolOrder'' should be a vector')
end
PolOrder = PolOrder(:)';    

% Make WinSize a row vector and make it an odd number.
if ( min(size(WinSize)) > 1 )
    error('''WinSize'' should be a vector')
end
WinSize = WinSize(:)';

if ( any(~(rem(WinSize,2))) )
    %   If some WinSizedow size is even
    %   correct this and warn user
    WinSize = unique(WinSize + ~(rem(WinSize,2)));
    str     = sprintf('%d ',WinSize);
    warning(sprintf('Some of the WinSizedow sizes were even and should be odd, now used: %s',str))
end    

% Make PDSlvs a row vector.
if ( min(size(PDSlvs)) > 1 )
    error('''PDSlvs'' should be a vector')
end
PDSlvs = PDSlvs(:)';    

%   ==================================================
%   =========   End of argument Checking =============
%   ==================================================


DeltaT          = Temps - Tcal;       % DeltaT is temperature difference with respect to
                                      % the calibration temperature.
testTemps       = setdiff(Temps,[TNOcv,Tcal]);
                                      % testTemps is a vector with temperatures to 
                                      % crossvalidate over. Note: the calibration
                                      % temperature is also left out.
CalIndex        = find( Temps == Tcal);

if ( model.mncn == 1 )
    [temp,MNSpec]       = mncn(squeeze(xSpectra(:,:,CalIndex)));
    [temp,MNy]          = mncn(yCal);
    
elseif ( model.auto == 1 )
    [temp,MNSpec,SSpec] = auto(squeeze(xSpectra(:,:,CalIndex)));
    [temp,Sy]           = auto(yCal);
    
end 
                      
iomega      = 0;         % Counter
for ( omega = WinSize )  % Crossvalidation over the window size in PDS.

    iomega  = iomega + 1;
    inu     = 0;            % Counter
    for ( nu = PDSlvs )     % Crossvalidation over the number of LVs in PDS.
    
          inu = inu + 1;                 
          if ( omega >= nu )
             %  The size of the window (omega) should at least be equal to the number
             %  internal latent vectors in PDS (nu). 
             
             itest = 0;                 % Counter
             for ( testT = testTemps )  % Crossvalidation over the test temperatures.
             
                TestIndex   = find ( testT == Temps );
                itest       = itest + 1;
                fprintf('Window Size: %g, Number of lvs in PDS: %g, Test temperature: %g, Polynomial orders: ',omega,nu,testT); 
                
                istd    = 0;
                for ( teststd = stds )  %  Leave out a standardisation sample.
                
                   istd         = istd + 1;
                   stdsa        = setdiff(stds,teststd);
                   [Pmatrices]  = tfb(xSpectra,stdsa,omega,nu,TestIndex,CalIndex,1);

                   ieta          = 0;       % Counter
                   for ( eta = PolOrder )   % Crossvalidate over polynomial order.
                      
                      ieta                  = ieta + 1;
                      Ptest                 = GetPmat(Pmatrices,Temps,testT,DeltaT,eta);
                      RecSpectra            = xSpectra(teststd,:,TestIndex) * Ptest; 
                      RecErrors             = RecSpectra - xSpectra(teststd,:,CalIndex);
                      
                      % Store reconstruction errors.
                      RecSS(inu,iomega,ieta,itest,istd) = RecErrors * RecErrors';
                                            
                      % Evaluate the multivariate model for the reconstructed spectrum.
                      if ( model.mncn == 1 )
                          RecSpectra    = scale(RecSpectra,MNSpec);
                      elseif ( model.auto == 1 )                          
                          RecSpectra    = scale(RecSpectra,MNSpec,SSpec);
                      end                          
                      yPRED         = RecSpectra * model.bvec;
                      if ( model.mncn == 1 )
                        Residual      = yCal(teststd) - rescale(yPRED,MNy);
                      elseif ( model.auto == 1 )                          
                        Residual      = yCal(teststd) - rescale(yPRED,MNy,Sy);
                      else
                        Residual      = yCal(teststd) - yPRED;
                      end                        
                      PreSS(inu,iomega,ieta,itest,istd) = Residual^2;
                      
                    end % Polynomial Order loop                               
                    
                end % Standardization Samples loop    
                fprintf('%d ',PolOrder);                     
                fprintf('\n');
            end 
                
        end % Test Temperatures loop
         
    end % Number of LV's in PDS loop.
   
end % Window Size of PDS loop.

% Make a table (comparable with Table II in reference) with RMSEC_CV's
for ( i = 1:inu ) 
    % first dimension: nu (#PDS latent variables) 
    for ( j = 1:iomega )
        % Second dimension: omega (Window Size in PDS)
        for ( k = 1:ieta )
            % Third dimension: eta (Order of interpolation polynomial)
            temp              = PreSS(i,j,k,:,:);
            RMSEC_CV(i,j,k)   = sqrt(mean(temp(:)));
        end
    end        
end