% Example script:
% Illustration of the use of the CPDS crossvalidation routine.

% Data:
%   The dataset used consists of data measured by Florian Wulfert. This
%   dataset is fully described in the publication:
%       F. Wlfert  W. Th. Kok and  A.K. Smilde, Anal. Chem. 70, 1761 (1998)
%   
% Speed considerations:
%   MATLAB 5.2.1
%   on a PC under WINNT ; Pentium II processor ; 128 Mb ; 266 MHz
%   the time to run this script is: 15 min 51 seconds.
%
%   on a PC under WIN95 ; Pentium II processor ; 96 Mb ; 200 MHz
%   the time to run this script is: 27 min 36 seconds.
%
%   MATLAB 6.0.0
%   on a PC under WIN-NT 4.0; Pentium III processor ; 466 Mb ; 500 MHz
%   the time to run this script is: 21 min 44 seconds.
%
% Remark:
%   This script cannot be run without the PLS-toolbox being
%   available (mncn(), pls() and stdgen()). The script is
%   tested with version 2.0 or 2.1 of the PLS-toolbox( Copyright:
%   Eigenvector Research, Inc. 1994-98)
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
clear all
close all
load swnirdata

%   Construct a PLS model at the central calibration temperature
%   for compound 1.
calsa   = [1:4,7,8,10,12,13,16:19];     % Only calibration samples are used in crossvalidation
yCal    = squeeze(concs(calsa,1,1));

xdata  = mncn(squeeze(specs(calsa,:,3))); 
ydata  = mncn(yCal);
[bvec] = pls(xdata,ydata,4,0);

%   This is the model-info needed in CPDScv()
model.bvec  = bvec(4,:)';
model.mncn  = 1;
model.auto  = 0;

%   Set the temperatures needed.
Temps = [30,40,50,60,70];   % Temperature at which measurements were performed.
TNOcv = [30,70];            % Temperature NOT used in crossvalidation
                            % Extreme temperature are left out.
Tcal  = [50];               % Calibration temperature.

%   Fill in the values of the CPDS meta-parameters over which to cross-validate.
PolOrder = [1,2,3];
WinSize  = [3,5,7];
PDSlvs   = [2,3,4];

%   Get the all spectra of the calibration samples.
calspecs    = specs(calsa,:,:);

%   Indicate which calibration samples will be used for standardization.
stds        = [1 3 6 7 8 11 13];

%   Call the CPDS crossvalidation routine.
[RMSECV,RecSS,PreSS,testTemps] = CPDScv(calspecs,yCal,Temps,TNOcv,Tcal,stds,model,PolOrder,WinSize,PDSlvs);

%   Display RMSECV output:
fprintf('Results of CPDS cross-validation');
fprintf('\n+++++++++++++++++++++++++++++++++++++++++++++++++++++\n')
fprintf('w = The Window Size\n');
fprintf('n = Polynomial Order\n');
fprintf('v = Internal number of PDS latent variables\n');
fprintf('\n+++++++++++++++++++++++++++++++++++++++++++++++++++++\n')
for ( iorder = 1:length(PolOrder) )
    fprintf('n = %2d    ',PolOrder(iorder));
    for ( iwin = 1:length(WinSize) )
        fprintf('w = %2d     ',WinSize(iwin)) 
    end    
    fprintf('\n')
    for ( ilv = 1:length(PDSlvs) )
        fprintf('v = %2d    ',PDSlvs(ilv));
        fprintf('%-10.3f ',RMSECV(ilv,:,iorder))
        fprintf('\n')
    end    
    fprintf('\n+++++++++++++++++++++++++++++++++++++++++++++++++++++\n')
end

%   Some (not all) plots of the reconstruction error are displayed.
%   These plots indicate whether a particular test sample gives
%   rise to  a large reconstruction error. It is also possible to inspect
%   whether a test sample tends to be well transferred 
%   at one temperature and badly transferred at another temperature.   
ilv     = 2;
iwin    = 2;
ipol    = 2;
for ( itemp = 1:length(testTemps))
    subplot(2,2,itemp)
    plot(stds,squeeze(RecSS(ilv,iwin,ipol,itemp,:)),'o-');
    xlabel('Test sample number')
    ylabel('Reconstruction error')
    title(sprintf('Temperature %-5.1f',testTemps(itemp)))
end    

%   Also diagnostic plots may be drawn