function [Pmat,PolCoef]=GetPmat(Pmatrices,Temps,TestTemp,DeltaT,PolOrder)
%
%   [Pmat,PolCoef]=GetPmat(Pmatrices,Temps,TestTemp,DeltaT,PolOrder)
%
%   Purpose:
%   Fit polynomial of order 'PolOrder' to the entries in the P-matrices.
%   In this way the P-matrix for the test temperature (TestTemp) is determined.
%   Only the Pmatrices for those temperatures specified in 'Temps' that 
%   are not equal to temperature in specified in 'TestTemp' are used to do this.
%
%   Parameters:
%   Pmatrices   -   (Nwave x Nwave x Ntemps) matrix with:
%                   dimension 1 and 2       : P-matrix (Nwave x Nwave)
%                   dimension 3 (1..Ntemps) : indicates temperature of at which
%   Temps       -   (1 x Ntemps) vector with temperatures at which
%                   spectra of the samples were measured.
%   TestTemp    -   scalar with validation temperature.
%   DeltaT      -   (1 x Ntemps) Temperature differences with respect to
%                   calibration temperatures.
%   PolOrder    -   scalar indicating the polynomial order of the polynomial 
%                   for iterpolation in the entries of the Pmatrices.
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
TestIndex       = find( Temps == TestTemp);
OtherIndex      = find( Temps ~= TestTemp);
start           = 1;
for ( index = OtherIndex )
    [rows,cols]     =   find( squeeze(Pmatrices(:,:,index)) ~= 0);
    if ( start ) 
        Result          = [rows,cols]; 
        start           = 0;
    else        
        indexMat        = [rows,cols];
        Result          = union(Result,indexMat,'rows'); 
    end        
end    
Trows           =   Result(:,1);
Tcols           =   Result(:,2);
FitT            =   DeltaT(OtherIndex)';
Nwave           =   size(Pmatrices,1);
Pmat            =   zeros(Nwave,Nwave);
for (i   =  1 : length(Trows))
    % Do for all non-zero elements in the P-matrix of the other temperatures.
    Pentries                = squeeze(Pmatrices(Trows(i),Tcols(i),OtherIndex));
    PolCoef(i,:)            = polyfit(FitT, Pentries,PolOrder);
    Pmat(Trows(i),Tcols(i)) = polyval(PolCoef(i,:),DeltaT(TestIndex)); 
end
