function [Pmatrices]=tfb(xSpectra,stdsa,WinSize,PDSlvs,TestIndex,CalIndex,CalIdentity)
%
%   [Pmatrices]=tfb(xSpectra,stdsa,WinSize,PDSlvs,TestIndex,CalIndex,CalIdentity)
%
%   purpose:
%   Creates a 3D tensor, called Pmatrices, using the measured spectra. A P-matrix
%   is a transformation matrix from one temperature to the calibration temperature
%   The calibration temperature is the temperature for which a calibration
%   model is available (see reference).
%
%   Parameters:
%   xSpectra    -   3D tensor with spectra of calibration samples 
%                   measured at different temperatures.
%                   first dimension:    samples
%                   second dimension:   channels (wavelengths) 
%                   third dimension:    temperatures
%   stdsa       -   Sample numbers (standardization samples) used for doing PDS.
%   WinSize     -   Window size used in PDS.
%   PDSlvs      -   Number of latent variables used in PDS.
%   TestIndex   -   Index of current test temperature in third dimension of xSpectra. 
%   CalIndex    -   Index of calibration temperature in third dimension of xSpectra. 
%   CalIdentity -   1   use the identity matrix at calibration temperature.
%                   0   don't use the identity matrix at calibration temperature.
%
%   Returned Parameters:
%   Pmatrices   -   Pmatrices(:,:,1): Pmatrix to transfer of spectra (stdsa) measured 
%                   at first temperature ( = specs(:,:,1)) to the calibration temperature.
%
%   Reference:
%   Optimizing Meta-parameters in Continuous Piecewise Direct Standardization
%   Hans K. Barring, Hans F.M. Boelens, Onno E. de Noord, Age K. Smilde
%   Applied Spectroscopy, to be published.
%
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================
[Nsamples,Nwave,Ntemps] =   size(xSpectra);  % Ntemps: Number of temperatures
                                             % Nwave : Number of wavelengths in spectrum.
tolerance               =   eps;             % This value has to be set lower than the
                                             % default value in pcstdgen.m, because
                                             % we want to have exactly: PDSlvs components.
                                             
for itemp = 1:Ntemps
    if ( itemp == CalIndex & CalIdentity )
        %   Set P-matrix to identity matrix for calibration temperature, if desired.
        Pmatrices(:,:,itemp) = eye(Nwave,Nwave);
    else
        if ( itemp ~= TestIndex ) 
            %   Calclulate P-matrix using PDS only for NON-Test temperatures.
            Pmatrices(:,:,itemp) = full(stdgen(xSpectra(stdsa,:,CalIndex),xSpectra(stdsa,:,itemp),WinSize,tolerance,PDSlvs)); 
        end
    end        
end